/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package ZEN_IDL.be;

import ZEN_IDL.ast.*;
import ZEN_IDL.be.symbolTableUtils.*;
import ZEN_IDL.be.helperUtils.*;
import util.*;

import java.util.*;
import java.io.*;

/*********************************************************
* Abstract class for marshaling
*
* @author Olav Haugan
*
* 14.10.01
*********************************************************/
public abstract class MQCAbstractMarshalHelper
{
	public MQCCodeGenHelper codeGenHelper; /* The code generator object help code */
	public MQCConfig config; /* The configuration options */
	public boolean mqcIOP;
	
	/*********************************************************
	* Constructor. Initializes variables.
	* 
	* @param Hashtable A method mapping object
	*********************************************************/
	public MQCAbstractMarshalHelper(MQCConfig config)
	{
		this.config = config;
		mqcIOP = false;
		codeGenHelper = new MQCCodeGenHelper(config);
		init();
	}

	/*********************************************************
	* Prints the marshalling code for 
	* either the client or server. 
	* 
	* @param PrintWriter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	* @param boolean true if code is for client, false
  *                if for server.
	*
	* @return int The size of buffer needed.
	*********************************************************/
	public abstract int printMarshal( MQCPrinter pw, InterfaceMethod iMethod, boolean Client );

	/*********************************************************
	* Prints the un-marshalling code for 
	* either the client or server. 
	* 
	* @param PrintWriter The outputstream.
	*        
	* @param InterfaceMethod The method
	*
	* @param boolean true if code is for client, false
  *                if for server.
	*********************************************************/
	public abstract void printUnMarshal( MQCPrinter pw, InterfaceMethod iMethod, boolean Client );

	/*********************************************************
	* Initialization function
	*********************************************************/
	public abstract void init();
	
	/*********************************************************
	* Returns true if we are marshaling MQCIOP, false otherwise
	*
	* @return boolean Returns true if we are marshaling MQCIOP, 
	* false otherwise
	*********************************************************/
	public boolean isMQCIOP()
	{
		return mqcIOP;
	}
}
