/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package ZEN_IDL.be;

import ZEN_IDL.ast.*;
import ZEN_IDL.be.symbolTableUtils.*;
import ZEN_IDL.be.helperUtils.*;
import util.*;

import java.io.*;
import java.util.*;
import java.lang.*;

/**
 * Generates makefile to be used to compile client
 * and server.
 *
 * @author Olav Haugan
 * @author Tarana Damania
 * @author A. David McKinnon
 * @author Thor Egil Skaug
 * @version 1.00
 */
public class MQCMakefileGen
{
  MQCConfig   config;        // config constraints (from the GUI)
  String      packageName;   // module name(s) from the IDL file
  PrintWriter pw;
  
  /*********************************************************
  * Constructor. Initializes variables. Creates a file to write to.
  * 
  * @param config       The MQCConfig configuration
  * @param packageName  Name of the IDL module
  *********************************************************/
  public MQCMakefileGen(MQCConfig config, String packageName)
  {
    this.config      = config;
    this.packageName = packageName;
    try
    {
      pw = new PrintWriter(new BufferedWriter(new FileWriter( "Makefile" )), false);
    }
    catch(java.io.IOException e)
    {
      System.out.println("Error creating makefile");
      System.exit(-1);
    }
  }
  
  /*********************************************************
  * Generates the makefile
  * 
  *********************************************************/
  public void generate()
  {
    pw.println("#Makefile generated by MicroQoSCORBA code generator tool");
    pw.println("#Make the necessary changes to the variables below");
    pw.println();
    pw.println("#Java compiler path");
    pw.println("JAVA  := $(JAVA_BIN)/java");
    pw.println("JAVAC := $(JAVA_BIN)/javac -target 1.1");
    pw.println("JAR   := $(JAVA_BIN)/jar");
    pw.println("#PC JAVAC := c:/jdk1.4.1/bin/javac -target 1.1");
    pw.println();
    pw.println("#Directory where you want the client and server classes to go");
    pw.println("CLIENT_DIR := ./client");
    pw.println("SERVER_DIR := ./server");
    pw.println();
    pw.println("#The path to were the MQC libraries are stored");
    pw.println("PATH  := $(MQC_HOME)");
    pw.println("MPATH := $(PATH)/mqc");
    pw.println("JPATH := ./mqc");
    pw.println();
    pw.println("#The Home for the ARKived binaries, etc.  (Note: ARK is passed in from");
    pw.println("#Makefile.mqcc and it is prepended with $(HOME). HARK_DOS is needed for SaJe)");
    pw.println("HARK     := $(MQC_ARK)/$(ARK)");
    pw.println("HARK_DOS := $(MQC_ARK_DOS)/$(ARK)");
    pw.println();
    pw.println("#Path to Third Party Tools, etc. (eg, aJile, TINI");
    pw.println("TOOLS_DIR := $(MQC_TOOLS)");
    pw.println("TINI_BIN  := $(TOOLS_DIR)/tini1.02e/bin");
    pw.println("AJILE_DIR := $(TOOLS_DIR)/aJile");
    pw.println();
    pw.println("CLIENT_SRC = \\");
    pw.println("\tClient.java \\");
    pw.println("#\tCLIENT_SRC end");
    pw.println();
    pw.println("#Server sources");
    pw.println("SERVER_SRC = \\");
    pw.println("\tServer.java \\");
    pw.println("#\tSERVER_SRC end");
    pw.println();
    pw.println("#Not neccessary to make any changes below this line");
    pw.println();
    pw.println("RM    := /bin/rm -f");
    pw.println("FIND  := /usr/bin/find");
    pw.println("M4    := /usr/bin/m4");
    pw.println("WC    := /usr/bin/wc -c");
    pw.println("XARGS := /usr/bin/xargs");
    pw.println("MKDIR := /bin/mkdir -p");
    pw.println("ECHO  := echo");
    pw.println("COPY  := /bin/cp");
    pw.println("TOUCH := /bin/touch");
    pw.println("SED   := /bin/sed");
    pw.println();
    pw.println("CP = -classpath .:$(PATH):$(PATH)/jni");
    pw.println("CLI_CP = $(CP):$(CLIENT_DIR)");
    pw.println("SRV_CP = $(CP):$(SERVER_DIR)");
    pw.println();

    // Define BCP for both J2SE and CLDC environments, just make sure that
    // it's commented out in the non-CLDC case (eg, print("#"))
    if (!config.getCLDCStatus()) {
      pw.println("BCP=");
      pw.print("#");
    }
    pw.println("BCP=-bootclasspath $(TOOLS_DIR)/J2mewtk/lib/midpapi.zip:$(TOOLS_DIR)/aJile/Runtime_cldc/Rts");
    pw.println();

    //pw.println("#Path to CDRStream library directory");
    //pw.print("CDR_PATH=$(PATH)/");
    
    //if(config.isInline())
      //pw.println("mqc/placeholder/");
    //else
      //pw.println("mqc/stream/");
    //pw.println();

    pw.println("JAVA_SRC = \\");
    pw.println("\tIORParser.java \\");
    pw.println("\tProfile.java \\");
    pw.println("\tTransport.java \\");
    pw.println("\tORB.java \\");
    pw.println("\tS_ORB.java \\");
    pw.println("\tPOA.java \\");
    pw.println("\tC_ORB.java \\");
    pw.println("\tDelegate.java \\");

    if (config.getDebugStatus())
       pw.println("\tmsgPrinter.java \\");


    Vector protocols = config.getServerProtocol();
     
    if(protocols.contains(Constants.GIOP_GIOP_STR))
    {
      pw.println("\tGIOPDelegate.java \\");
    }
    
    if (config.getSecurityCipherStatus()) {
      // pw.println("\tCipher.java \\");  // no macros for now
      pw.println("\tBlockCipher.java \\");
      // pw.println( config.getSecurityCipher() + ".java \\");
    }

    if(config.getUDPStatus())
    {
      pw.println("\tS_UDPTransport.java \\");
      pw.println("\tS_UDPGenericHandler.java \\");
      pw.println("\tUDPGenericTransport.java \\");
      
      if(config.getUnreliableUDPStatus())
      {
         pw.println("\tS_UDPHandler.java \\");
         pw.println("\tUDPTransport.java \\");
      }

      if(config.getGoBackNUDPStatus())
      {
         pw.println("\tS_UDPGoBackNHandler.java \\");
         pw.println("\tUDPGoBackNTransport.java \\");
      }
      
      if(config.getStopNWaitUDPStatus())
      {      
      pw.println("\tS_UDPStopNWaitHandler.java \\");
      pw.println("\tUDPStopNWaitTransport.java \\");
      }  
    } 
    else 
    {
      pw.println("\tTCPTransport.java \\");
      pw.println("\tS_TCPHandler.java \\");
      pw.println("\tS_TCPTransport.java \\");
    }
    pw.println("#\tJAVA_SRC end");
    pw.println();

    pw.println("##-- where to look for the *.java.m4 files --");
    pw.print("vpath %.java.m4 $(MPATH):$(MPATH)/protocols:$(MPATH)/transports:$(MPATH)/ior:$(MPATH)/util");
    
    if (config.getSecurityCipherStatus()) 
    {
      pw.print(":$(MPATH)/security:$(MPATH)/security/cipher");
    }
    if (config.getUDPStatus()) 
    {
      pw.print(":$(MPATH)/udp");
    }
    if (config.getDebugStatus()) 
    {
      pw.println("\n");
    }

    pw.println("##-- where to look for the *.java files --");
    pw.print("vpath %.java    $(JPATH):$(JPATH)/protocols:$(JPATH)/transports:$(JPATH)/ior:$(JPATH)/util");
    if (config.getSecurityCipherStatus()) {
      pw.print(":$(JPATH)/security:$(JPATH)/security/cipher");
    }
    if (config.getUDPStatus()) {
      pw.print(":$(JPATH)/udp");
    }
    pw.println("\n");

    pw.println(".PHONY: all jar linux tini saje ark tark sark clean realclean stats");
    pw.println();

    pw.println("all: $(CLIENT_DIR)/Client.class $(SERVER_DIR)/Server.class");
    pw.println();

    pw.println("jar: Client.jar Server.jar");
    pw.println();

    pw.println("linux: jar");
    pw.println();
    
    pw.println("##-- append tini-specific classpath info when building tini targets --");
    pw.println("tini: CLI_CP = $(CLI_CP):$(TINI_BIN)/tiniclasses.jar");
    pw.println();

    pw.println("tini: Client.tini Server.tini");
    pw.println();

    pw.println("##-- append SaJe-specific classpath info when building SaJe targets --");
    pw.println("saje: CLI_CP = $(CLI_CP):$(AJILE_DIR)/Runtime_cldc/Rts");
    pw.println();

    pw.println("saje: .SaJeClient .SaJeServer");
    pw.println();

    pw.println("%.java : %.java.m4");
    pw.println("\t$(MKDIR) $(patsubst $(PATH)%, .%, $(dir $<))");
    pw.println("\t$(RM) $(patsubst $(PATH)%.java.m4, .%.class, $<)");
    pw.println("\t$(M4) $< > $(patsubst $(PATH)%.m4, .%, $<)");
    pw.println();

    pw.println("%.tini : %.jar");
    pw.println("\t$(JAVA) -classpath $(TINI_BIN)/tini.jar TINIConvertor -f $^ -d $(TINI_BIN)/tini.db -o $@");
    pw.println();

    pw.println("$(CLIENT_DIR)/mqc/Config.class: Config.java");
    pw.println("\t$(JAVAC) $(BCP) -classpath . -d $(CLIENT_DIR) $^");
    pw.println();

    //pw.println("$(CLIENT_DIR)/mqc/CDRInputStream.class: $(CDR_PATH)/CDRInputStream.java");
    //pw.println("\t$(JAVAC) $(BCP) -classpath . -d $(CLIENT_DIR) $^");
    //pw.println();

    //pw.println("$(CLIENT_DIR)/mqc/CDROutputStream.class: $(CDR_PATH)/CDROutputStream.java");
    //pw.println("\t$(JAVAC) $(BCP) -classpath . -d $(CLIENT_DIR) $^");
    //pw.println();

    pw.println("$(SERVER_DIR)/mqc/Config.class: Config.java");
    pw.println("\t$(JAVAC) $(BCP) -classpath . -d $(SERVER_DIR) $^");
    pw.println();

    //pw.println("$(SERVER_DIR)/mqc/CDRInputStream.class: $(CDR_PATH)/CDRInputStream.java");
    //pw.println("\t$(JAVAC) $(BCP) -classpath . -d $(SERVER_DIR) $^");
    //pw.println();

    //pw.println("$(SERVER_DIR)/mqc/CDROutputStream.class: $(CDR_PATH)/CDROutputStream.java");
    //pw.println("\t$(JAVAC) $(BCP) -classpath . -d $(SERVER_DIR) $^");
    //pw.println();

    pw.println("Client.java: Client.java.m4");
    pw.println("\t$(M4) $< > $@");
    pw.println();

    //pw.println("$(CLIENT_DIR)/Client.class: $(CLIENT_SRC) $(JAVA_SRC) $(CLIENT_DIR)/mqc/Config.class $(CLIENT_DIR)/mqc/CDRInputStream.class $(CLIENT_DIR)/mqc/CDROutputStream.class");
    pw.println("$(CLIENT_DIR)/Client.class: $(CLIENT_SRC) $(JAVA_SRC) $(CLIENT_DIR)/mqc/Config.class");
    pw.println("\t$(JAVAC) $(BCP) $(CLI_CP) -d $(CLIENT_DIR) Client.java");
    pw.println();

    pw.println("Client.jar: $(CLIENT_DIR)/Client.class Client.MANIFEST.MF");
    pw.print("\tcd $(CLIENT_DIR); ");
    pw.println("\t$(JAR) -cvmf ../Client.MANIFEST.MF ../Client.jar -C .. config.mqcc *.class mqc " + packageName);
    pw.println();

    pw.println("Client.MANIFEST.MF:");
    pw.println("\t$(ECHO) \"Main-Class: Client\" > $@");
    pw.println();

    pw.println("Server.java: Server.java.m4");
    pw.println("\t$(M4) $< > $@");
    pw.println();

    //pw.println("$(SERVER_DIR)/Server.class: $(SERVER_SRC) $(JAVA_SRC) $(SERVER_DIR)/mqc/Config.class $(SERVER_DIR)/mqc/CDRInputStream.class $(SERVER_DIR)/mqc/CDROutputStream.class");
    pw.println("$(SERVER_DIR)/Server.class: $(SERVER_SRC) $(JAVA_SRC) $(SERVER_DIR)/mqc/Config.class");
    pw.println("\t$(JAVAC) $(BCP) $(SRV_CP) -d $(SERVER_DIR) Server.java");
    pw.println();

    pw.println("Server.jar: $(SERVER_DIR)/Server.class Server.MANIFEST.MF");
    pw.print("\tcd $(SERVER_DIR); ");
    pw.println("\t$(JAR) -cvmf ../Server.MANIFEST.MF ../Server.jar -C .. config.mqcc *.class mqc " + packageName);
    pw.println();

    pw.println("Server.MANIFEST.MF:");
    pw.println("\t$(ECHO) \"Main-Class: Server\" > $@");
    pw.println();

    pw.println("SaJeClient.ajp: SAJE_CLIENT.AJP");
    pw.println("\t$(SED) -e \"s#OUTPUT_DIR#$(HARK)_Client.saje#\" < $< > $@");
    pw.println();
    
    pw.println(".SaJeClient: SaJeClient.ajp Client.jar");
    pw.println("\t-$(JAVA) -cp $(AJILE_DIR)/JemBuilder/jembuilder.jar -Dlax.root.install.dir=$(AJILE_DIR) com.ajile.jembuilder.JemBuilder $(AJILE_DIR)/JemBuilder/library.properties SaJeClient.ajp");
    pw.println("\t$(ECHO) \"Converting Unix path names to DOS path names...\"");
    pw.println("\t$(COPY) $(HARK)_Client.saje/SaJeClient.sod $(HARK)_Client.saje/SaJeClient.sod.unix");
    pw.println("\t$(SED) -e \"s#$(HARK)#$(HARK_DOS)#\" < $(HARK)_Client.saje/SaJeClient.sod.unix > $(HARK)_Client.saje/SaJeClient.sod");
    pw.println("\t$(COPY) $(HARK)_Client.saje/build.jcf $(HARK)_Client.saje/build.jcf.unix");
    pw.println("\t$(SED) -e \"s#$(HARK)#$(HARK_DOS)#\" < $(HARK)_Client.saje/build.jcf.unix > $(HARK)_Client.saje/build.jcf");
    pw.println("\t$(TOUCH) $@");
    pw.println();
    
    pw.println("SaJeServer.ajp: SAJE_SERVER.AJP");
    pw.println("\t$(SED) -e \"s#OUTPUT_DIR#$(HARK)_Server.saje#\" < $< > $@");
    pw.println();
    
    pw.println(".SaJeServer: SaJeServer.ajp Server.jar");
    pw.println("\t-$(JAVA) -cp $(AJILE_DIR)/JemBuilder/jembuilder.jar -Dlax.root.install.dir=$(AJILE_DIR) com.ajile.jembuilder.JemBuilder $(AJILE_DIR)/JemBuilder/library.properties SaJeServer.ajp");
    pw.println("\t$(ECHO) \"Converting Unix path names to DOS path names...\"");
    pw.println("\t$(COPY) $(HARK)_Server.saje/SaJeServer.sod $(HARK)_Server.saje/SaJeServer.sod.unix");
    pw.println("\t$(SED) -e \"s#$(HARK)#$(HARK_DOS)#\" < $(HARK)_Server.saje/SaJeServer.sod.unix > $(HARK)_Server.saje/SaJeServer.sod");
    pw.println("\t$(COPY) $(HARK)_Server.saje/build.jcf $(HARK)_Server.saje/build.jcf.unix");
    pw.println("\t$(SED) -e \"s#$(HARK)#$(HARK_DOS)#\" < $(HARK)_Server.saje/build.jcf.unix > $(HARK)_Server.saje/build.jcf");
    pw.println("\t$(TOUCH) $@");
    pw.println();

    pw.println("ark: linux");
    pw.println("\t$(COPY) Client.jar  $(HARK)_Client.jar");
    pw.println("\t$(COPY) Server.jar  $(HARK)_Server.jar");
    pw.println("\techo \"Computing Client side stats...\"                 >  $(HARK)_stats");
    pw.println("\t$(FIND) $(CLIENT_DIR) -name \"*class\" | $(XARGS) $(WC) >> $(HARK)_stats");
    pw.println("\techo \"Computing Server side stats...\"                 >> $(HARK)_stats");
    pw.println("\t$(FIND) $(SERVER_DIR) -name \"*class\" | $(XARGS) $(WC) >> $(HARK)_stats");
    pw.println();

    pw.println("tark: tini");
    pw.println("\t$(COPY) Client.tini $(HARK)_Client.tini");
    pw.println("\t$(COPY) Server.tini $(HARK)_Server.tini");
    pw.println("\techo \"Computing TINI Client side stats...\"            >  $(HARK)_stats_tini");
    pw.println("\t$(FIND) $(CLIENT_DIR) -name \"*class\" | $(XARGS) $(WC) >> $(HARK)_stats_tini");
    pw.println("\techo \"Computing TINI Server side stats...\"            >> $(HARK)_stats_tini");
    pw.println("\t$(FIND) $(SERVER_DIR) -name \"*class\" | $(XARGS) $(WC) >> $(HARK)_stats_tini");
    pw.println("\techo \"Computing TINI stats...\"                        >> $(HARK)_stats_tini");
    pw.println("\t$(WC) Client.tini Server.tini                           >> $(HARK)_stats_tini");
    pw.println();

    pw.println("sark: saje");
    pw.println("\t$(MKDIR) $(HARK)_Client.saje");
    pw.println("\t$(MKDIR) $(HARK)_Server.saje");
    pw.println("\t$(COPY) Client.jar  $(HARK)_Client.saje/Client.jar");
    pw.println("\t$(COPY) Server.jar  $(HARK)_Server.saje/Server.jar");
    pw.println("\techo \"Computing SaJe Client side stats...\"                >  $(HARK)_stats_saje");
    pw.println("\t$(FIND) $(CLIENT_DIR) -name \"*class\" | $(XARGS) $(WC)     >> $(HARK)_stats_saje");
    pw.println("\techo \"Computing SaJe Server side stats...\"                >> $(HARK)_stats_saje");
    pw.println("\t$(FIND) $(SERVER_DIR) -name \"*class\" | $(XARGS) $(WC)     >> $(HARK)_stats_saje");
    pw.println("\techo \"Computing SaJe BIN stats...\"                        >> $(HARK)_stats_saje");
    pw.println("\t$(FIND) $(HARK)_Client.saje -name \"*bin\" | $(XARGS) $(WC) >> $(HARK)_stats_saje");
    pw.println("\t$(FIND) $(HARK)_Server.saje -name \"*bin\" | $(XARGS) $(WC) >> $(HARK)_stats_saje");
    pw.println();

    pw.println("clean:");
    pw.println("\t$(RM) Client.jar Server.jar");
    pw.println("\t$(RM) Client.tini Server.tini");
    pw.println("\t$(RM) SaJeClient.ajp .SaJeClient");
    pw.println("\t$(RM) SaJeServer.ajp .SaJeServer");
    pw.println("\t$(RM) Client.java Server.java");
    pw.println("\t$(FIND) . -name \"*.class\"       | $(XARGS) $(RM)");
    pw.println("\t$(FIND) $(JPATH) -name \"*.java\" | $(XARGS) $(RM)");
    pw.println();

    pw.println("realclean: clean");
    pw.println("\t$(FIND) . -name \"*.jar\"     | $(XARGS) $(RM)");
    pw.println("\t$(FIND) . -name \"*.tini\"    | $(XARGS) $(RM)");
    pw.println("\t$(FIND) . -name \"*.sod\"     | $(XARGS) $(RM)");
    pw.println("\t$(RM) Client.MANIFEST.MF Server.MANIFEST.MF");
    pw.println("\t$(RM) *.corbaloc *.ior");
    pw.println("\t$(RM) -R $(CLIENT_DIR)");
    pw.println("\t$(RM) -R $(SERVER_DIR)");
    pw.println("\t$(RM) -R $(JPATH)");
    pw.println();

    pw.println("stats:");
    pw.println("\techo \"Computing Client side stats...\"");
    pw.println("\t$(FIND) $(CLIENT_DIR) -name \"*class\" | $(XARGS) $(WC)");
    pw.println("\techo \"Computing Server side stats...\"");
    pw.println("\t$(FIND) $(SERVER_DIR) -name \"*class\" | $(XARGS) $(WC)");
    pw.println("\techo \"Computing TINI stats...\"");
    //pw.println("\t$(FIND) . -name \"*tini\" | $(XARGS) $(WC)");
    pw.println("\t$(WC) Client.tini Server.tini");
    pw.println();

    pw.println("#End of file");
    pw.flush();
    pw.close();
  }
}
