/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package setup;
import java.io.*;
import java.util.*;

/**
 * This class exports the environment variables that
 * is specified in the setup program. With this
 * class you can either export bash or csh scripts
 */
public class export
{
private String JAVA_HOME;
private String MQC_HOME;
private String MQC_TOOLS;
private String MQC_ARK;
private String MQC_ARK_DOS;

    /**
     * Constructor for class export
     * @param JAVA_HOME The directory where java is installed, JAVA_BIN is calculated from here
     * @param MQC_HOME The directory where MQC is installed.
     * @param MQC_TOOLS The directory where third party tools are installed, eg. AJILE or TINI
     */
    public export(String JAVA_HOME, String MQC_HOME, String MQC_TOOLS, String MQC_ARK, String MQC_ARK_DOS)
    {
        this.JAVA_HOME   = JAVA_HOME;
        this.MQC_HOME    = MQC_HOME;
        this.MQC_TOOLS   = MQC_TOOLS;
        this.MQC_ARK     = MQC_ARK;
        this.MQC_ARK_DOS = MQC_ARK_DOS;
    }

    /**
     * Exports the settings to a bash file
     * @param fName The path and filename to write settings
     */
    public void export_bash(File fName)
    {
          FileOutputStream out1 = null;
        try
                {
                  out1 = new FileOutputStream(fName);
                } catch (IOException e)
                {
                  System.out.println("**Error: could not open file for writing");
                }
        PrintWriter out = new PrintWriter(out1);
        out.print("#!/bin/bash\n");
        out.print("#Environment variables for MQC! - BASH\n");
        out.print("JAVA_BIN="    + JAVA_HOME + File.separator + "bin\n");
        out.print("MQC_HOME="    + MQC_HOME    + "\n");
        out.print("MQC_TOOLS="   + MQC_TOOLS   + "\n");
        out.print("MQC_ARK="     + MQC_ARK     + "\n");
        out.print("MQC_ARK_DOS=" + MQC_ARK_DOS + "\n");
        out.print("export JAVA_BIN MQC_HOME MQC_TOOLS MQC_ARK MQC_ARK_DOS\n");
        out.flush();
    }

    /**
     * Exports settings to a csh file
     * @param fName The path and filename to write settings.
     */
    public void export_csh(File fName)
    {
        FileOutputStream out1 = null;
                   try
                   {
                     out1 = new FileOutputStream(fName);
                   } catch (IOException e)
                   {
                     System.out.println("**Error: could not open file for writing");
                   }
        PrintWriter out = new PrintWriter(out1);
        out.print("#!/bin/csh\n");
        out.print("#Environment variables for MQC! - CSH\n");
        out.print("setenv JAVA_BIN "    + JAVA_HOME + File.separator + "bin\n");
        out.print("setenv MQC_HOME "    + MQC_HOME    + "\n");
        out.print("setenv MQC_TOOLS "   + MQC_TOOLS   + "\n");
        out.print("setenv MQC_ARK "     + MQC_ARK     + "\n");
        out.print("setenv MQC_ARK_DOS " + MQC_ARK_DOS + "\n");
        out.print("if (\"$?LD_LIBRARY_PATH\") then\n");
        out.print("  setenv LD_LIBRARY_PATH $MQC_HOME/jni:$LD_LIBRARY_PATH\n");
        out.print("else\n");
        out.print("  setenv LD_LIBRARY_PATH $MQC_HOME/jni\n");
        out.print("endif\n");


        out.flush();
    }
}
