/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

//package timing;

import java.io.*;
import org.omg.CORBA.*;

public class Client 
{
    public static void main( String args[] ) 
    {
        if( args.length != 1 ) 
        {
            System.out.println( "Usage: jaco timing.Client <ior_file>" );
            System.exit( 1 );
        }

        try 
        {
            File f = new File( args[ 0 ] );

            //check if file exists
            if( ! f.exists() )
            {
                System.out.println("File " + args[0] + " does not exist.");
                System.exit( -1 );
            }
            
            //check if args[0] points to a directory
            if( f.isDirectory() )
            {
                System.out.println("File " + args[0] + " is a directory.");
                System.exit( -1 );
            }

            // Get the number of iterations
            int iterations = 1000;  // The default number of iterations
            String cntStr = System.getProperty("cnt");
            if (cntStr != null) {
                iterations = Integer.parseInt(cntStr);
            } else {
                System.out.println("Undefined 'cnt' property, using default");
            }
            System.out.println("Iteration 'cnt' set to " + iterations);

            // initialize the ORB.
            ORB orb = ORB.init( args, null );

            BufferedReader br = new BufferedReader( new FileReader( f ));

            // get object reference from command-line argument file
            org.omg.CORBA.Object obj = orb.string_to_object( br.readLine() );

            br.close();

            // and narrow it to Timing.Foo
            // if this fails, a BAD_PARAM will be thrown
            timing.foo _foo = timing.fooHelper.narrow( obj );

            long startTime = 0;
            long totalTime = 0;
            System.out.println("Ready to start the timing loop");
            ShowMemory();

            startTime = System.currentTimeMillis();
            for(int i = 0; i < iterations; i++) {
                 _foo.bar(i);
            }
            totalTime = (System.currentTimeMillis() - startTime);

            long time = (1000 * totalTime) / iterations;
            System.out.println("*** Total time (s): "+ totalTime/1000 +" *** Average time (us):\t" + time);

            ShowMemory();
        }
        catch( Exception ex ) 
        {
            System.err.println( ex );
        }
    }

  //-- Memory usage for Java --
  static void ShowMemory() {
    long tMem, fMem, uMem, fMem1, uMemMax, fMemMax;
    Runtime runtime = Runtime.getRuntime();

    tMem    = runtime.totalMemory();
    fMem    = runtime.freeMemory();
    fMemMax = fMem;
    if (tMem == fMem) {
      // TINI hack, tMem and fMem are reported as equal
      tMem = 334240;  // MAGIC NUMBER appears to be the proper value
    }
    uMem    = tMem - fMem;
    System.out.println("c.Memory (t/f/u): \t"+tMem+" \t"+fMem+" \t"+uMem);

    System.gc();
    try {
      Thread.sleep(500);
    } catch (InterruptedException ie) {}

    for (int i = 0;  i < 100;  i++) {
      try {
        Thread.sleep(500);
      } catch (InterruptedException ie) {}

      fMem1   = runtime.freeMemory();

      ;

      if (fMem1 > fMemMax ) {
        fMemMax = fMem1;
      } else {
        break;
      }
    }
    uMemMax = tMem - fMemMax;

    System.out.println("c.Memory (t/f/u): \t"+tMem+" \t"+fMemMax+" \t"+uMemMax);
  }

}

