/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package mqc.protocols;

/**
 * @author Kim Christian Swenson
 */
public class GIOPHeader_1_1 extends GIOPHeader
{
   protected byte m_flags;

   public GIOPHeader_1_1()
   {
      //Initialize defaults
      m_msgType = GIOPMessage.REQUEST_MSG;
      m_msgSize = 0;
      m_littleEndian = false;
      m_major = 1;
      m_minor = 1;
      m_flags = (byte)0x00;
   }

   public boolean isBigEndian()
   {
      return (m_flags & 0x01) == 0;
   }

   public void setBigEndian( boolean bigEndian )
   {
      super.setBigEndian( bigEndian );
      m_flags = bigEndian ? (byte)(m_flags & 0xFE) : (byte)( m_flags | 0x01);
   }

   public void deMarshal( CDRInputStream in ) throws RuntimeException
   {
      boolean isGIOP = 
            in.readOctet() == GMAGIC &&
            in.readOctet() == IMAGIC &&
            in.readOctet() == OMAGIC &&
            in.readOctet() == PMAGIC;

      if ( ! isGIOP )
      {
         throw new RuntimeException("Not a GIOP header.");
      }

      boolean isMajorOne = in.readOctet() == m_major;
      boolean isMinorZero = in.readOctet() == m_minor;

      if ( ! isMajorOne || ! isMinorZero )
      {
         throw new RuntimeException("Wrong GIOP version, should be " + m_major
               + "." + m_minor );
      }

      m_flags = in.readOctet();

      //whether least significant bit is 1
      m_littleEndian = ( (m_flags & 0x01) == 0x01);

      in.setBigEndian( ! m_littleEndian );

      m_msgType = in.readOctet();

      m_msgSize = in.readUnsignedLong();
   }

   public void marshal( CDROutputStream out )
   {
      out.writeOctet( GMAGIC );
      out.writeOctet( IMAGIC );
      out.writeOctet( OMAGIC );
      out.writeOctet( PMAGIC );
      out.writeOctet( m_major );
      out.writeOctet( m_minor );
      out.writeOctet( m_flags );
      out.setBigEndian( ! m_littleEndian );
      out.writeOctet( m_msgType );
      out.writeUnsignedLong( m_msgSize );
   }

}
