/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package mqc.protocols;

/**
 * 
 * The following struct should be marshaled using
 * the CDR transfer syntax.
 *
 * struct RequestHeader_1_1 {
 *    IOP::ServiceContextList  service_context;
 *    unsigned long            request_id;
 *    boolean                  response_expected;
 *    octet                    reserved[3];
 *    sequence<octet>          object_key;
 *    string                   operation;
 *    CORBA::OctetSeq          requesting_principal;
 *
 */
public class GIOPRequestHeader_1_1 extends GIOPRequestHeader
{
   protected byte[] m_requesting_principal;

   public GIOPRequestHeader_1_1()
   {
      m_requesting_principal = new byte[0];
   }

   public void deMarshal( CDRInputStream in ) throws RuntimeException
   {
      // IOP::ServiceContextList service_context 
      // read length of contextlist
      int n = in.readUnsignedLong();
      // SKIP contextlist
      for ( int i=0; i<n; i++) 
      {
         //SeriviceId context_id
         in.readUnsignedLong();
         //sequence<octet> context_data
         in.readOctetSequence();
      }

      // unsigned long request_id
      m_reqID = in.readUnsignedLong();

      // boolean response_expected
      m_oneWay = ! in.readBoolean();

      // octet reserved[3]
      in.readOctet();
      in.readOctet();
      in.readOctet();

      //sequence<octet> object_key
      m_objectKey = in.readOctetSequence();

      // string operation
      m_operation = in.readString();

      // CORBA::OctetSeq requesting_principal
      in.readOctetSequence();

   }

   public void marshal( CDROutputStream out )
   {
      // IOP::ServiceContextList service_context
      out.writeUnsignedLong( 0 );

      // unsigned long request_id
      out.writeUnsignedLong( m_reqID );

      // boolean response_expected
      out.writeBoolean( ! m_oneWay );

      // octet reserved[3]
      out.writeOctet( (byte)0 );
      out.writeOctet( (byte)0 );
      out.writeOctet( (byte)0 );

      // sequence<octet> object_key
      out.writeOctetSequence( m_objectKey );

      // string operation
      out.writeString( m_operation );

      // CORBA::OctetSeq requesting_principal
      out.writeOctetSequence( m_requesting_principal );
   }

}

