/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

package mqc.protocols;

/**
 * Hello.
 * 
 * typedef short AddressingDisposition;
 * const short KeyAddr = 0;
 * const short ProfileAddr = 1;
 * const short ReferenceAddr = 2;
 *
 * struct IORAddressingInfo {
 *    unsigned long            selected_profile_index;
 *    IOP::IOR                 ior;
 * };
 *
 * union TargetAddress switch (AddressingDisposition) {
 *    case KeyAddr:            sequence<octet> object_key;
 *    case ProfileAddr:        IOP::TaggedProfile profile;
 *    case ReferenceAddr:      IORAddressingInfo ior;
 * };
 *
 * // The following struct should be marshaled using
 * // the CDR transfer syntax.
 * 
 * struct RequestHeader_1_2 {
 *    IOP::ServiceContextList  service_context;
 *    unsigned long            request_id;
 *    boolean                  response_expected;
 *    octet                    reserved[3];
 *    sequence<octet>          object_key;
 *    string                   operation;
 *    CORBA::OctetSeq          requesting_principal;
 * };
 *
 */
public class GIOPRequestHeader_1_2 extends GIOPRequestHeader
{
   public static final short KEYADDR       = 0;
   public static final short PROFILEADDR   = 1;
   public static final short REFERENCEADDR = 2;

   public static final byte SYNCSCOPE_WITH_TRANSPORT = (byte) 0x00;
   public static final byte SYNCSCOPE_WITH_SERVER    = (byte) 0x01;
   public static final byte SYNCSCOPE_WITH_TARGET    = (byte) 0x03;

   protected byte m_responseFlags;

   public GIOPRequestHeader_1_2()
   {
      m_responseFlags = SYNCSCOPE_WITH_TARGET;
   }

   public void deMarshal( CDRInputStream in ) throws RuntimeException
   {
      // unsigned long request_id
      m_reqID = in.readUnsignedLong();

      // octet response_flags
      m_responseFlags = in.readOctet();
      switch ( m_responseFlags )
      {
         case SYNCSCOPE_WITH_TRANSPORT:
            m_oneWay = true; break;
         case SYNCSCOPE_WITH_SERVER:
         case SYNCSCOPE_WITH_TARGET:
            m_oneWay = false; break;
         default:
            throw new RuntimeException( "Value of 'response_flags' is invalid." );
      }

      // octet reserved[3]
      in.readOctet();
      in.readOctet();
      in.readOctet();

      // TargetAddress target
      
      // get discriminant
      short dis = in.readShort();
      if ( dis != KEYADDR )
      {
         throw new RuntimeException( "TargetAddress discriminant " + dis + " not supported." );
      }
      
      // case KeyAddr: sequence<octet> object_key
      m_objectKey = in.readOctetSequence();

      // string operation
      m_operation = in.readString();

      // IOP::ServiceContextList service_context 
      // read length of contextlist
      int n = in.readUnsignedLong();

      // SKIP contextlist
      for ( int i=0; i<n; i++) 
      {
         //SeriviceId context_id
         in.readUnsignedLong();

         //sequence<octet> context_data
         in.readOctetSequence();
      }

   }

   public void marshal( CDROutputStream out )
   {
      // unsigned long request_id
      out.writeUnsignedLong( m_reqID );

      // octet response_flags
      out.writeOctet( m_responseFlags );

      // octet reserved[3]
      out.writeOctet( (byte) 0x00 );
      out.writeOctet( (byte) 0x00 );
      out.writeOctet( (byte) 0x00 );

      // TargetAddress target

      // AddressingDisposition
      out.writeShort( KEYADDR );
      
      // case KeyAddr: sequence<octet> object_key
      out.writeOctetSequence( m_objectKey );

      // string operation
      out.writeString( m_operation );

      // IOP::ServiceContextList service_context
      out.writeUnsignedLong( 0 );

   }
}
