/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

include(`Config_macros.m4')

package mqc.transports;

import mqc.*;
import mqc.protocols.*;
import mqc.holders.*;

import java.io.*;
import java.util.*;
__J2SE__(
import java.net.*;
)
__CLDC__(
import javax.microedition.io.Connection;
import javax.microedition.io.Connector;
import javax.microedition.io.StreamConnection;
)
__SECURITY__(
import mqc.security.GeneralSecurityException;
)
__SECURITY_MD__(
import mqc.security.DigestException;
)
__SECURITY_MAC__(
import mqc.security.ShortBufferException;
)

/**
 * TCP handler class
 *
 * @author Olav Haugan
 * @author David McKinnon
 * @author Kim Christian Swenson
 */
public class S_TCPHandler implements java.lang.Runnable
{

   private CDRInputStream m_in;
   private CDROutputStream m_out;

   private int msgHdrSize;

   public Thread activity = new Thread(this);
   __J2SE__(
   private Socket client;
   )
   __CLDC__(
   private StreamConnection client;
   )
        private DataInputStream in;
   private DataOutputStream out;
   private S_ORB orb;
   __TEMPSPACE__(
   int maxReqID;
   )

   private byte[] m_recvBuf= null;
   private byte[] m_sendBuf = null;

        private int msgSize=0;                          // used in both send() and run()
   __SECURITY__(
   private int padOffset=0;         //sets the padding size for encrypted msg.
   )
   __SECURITY_MAC__(
   private int macOffset=0;         //sets the offset for mac
   )
   __SECURITY_MD__(
   __SECURITY_MD_NOT_HMAC__(
   private int mdOffset=0;          //sets the offset for md
   ))
                
   /**
    * Constructor. Initializes variables.
    */
   __J2SE__(
   public S_TCPHandler(Socket client, S_ORB orb) throws IOException
   )
   __CLDC__(
   public S_TCPHandler(StreamConnection client, S_ORB orb) throws IOException
   )
   {
      __DEBUG__(System.err.println("S_TCPHandler()");)
      this.client = client;
      this.orb = orb;
      //Setting the size of the receive buffer
      
      __TEMPSPACE__(
      maxReqID = -1;
      )
      __J2SE__(
      in = new DataInputStream(client.getInputStream());
      out = new DataOutputStream(client.getOutputStream());
      )
      __CLDC__(
      in = new DataInputStream(client.openDataInputStream());
      out = new DataOutputStream(client.openDataOutputStream());
      )
      __FIX_SIZE__(
      /* FIX_SIZE BEGIN */
      msgSize = Config.intPacketSize;
      __SECURITY_MD__(
      __SECURITY_MD_NOT_HMAC__(    // do not add MD len if the MD is for
      mdOffset = msgSize;
      msgSize += ORB.msgDigestLen;  // an HMAC (ie, no double counting)
      ))
      __SECURITY_MAC__(
      macOffset = msgSize;
      msgSize += ORB.msgMacLen;
      )
      __SECURITY__(
      padOffset = msgSize; //sets the padding offset (no double padding!!)
      msgSize = orb.cipherDecrypt.getOutputSize(msgSize);
      )
      m_recvBuf = new byte [ msgSize ];
      m_sendBuf = new byte [ msgSize ];
      m_out = new CDROutputStream( msgSize , 0, false );
      m_in = new CDRInputStream( m_recvBuf, 0, false );
      /* FIX_SIZE END */
      )

      __VAR_SIZE__(
      // Initial variable size
      m_sendBuf = new byte[0];
      m_recvBuf = new byte[64];
      m_out = new CDROutputStream( 64, 0, false );
      m_in = new CDRInputStream( m_recvBuf, 0, false);
      )

      activity.start();
   }

   /**
    * Thread run function. Read in data from socket
    */
   public void run()
   {
      Marshalable msg;

      __DEBUG1__(int nT = 0; long[] sTimes = new long[1000]; int[] reqIDs = new int[1000];)
      __DEBUG1__(System.err.println(" run");)

      while(S_ORB.keepRunning)
      {
         msg = null;

         __VAR_SIZE__(
         /* VAR_SIZE BEGIN */
         try 
         {
            // read first 4 bytes of header
            in.readFully(m_recvBuf, 0, 4);

            // figure out header size
            msgHdrSize = Message.getHdrSize(m_recvBuf);

            // read rest of header
            in.readFully(m_recvBuf, 4, msgHdrSize-4);

            // Find out whether message uses bigEndian here!
            boolean bigEndian = Message.isBigEndian( m_recvBuf );

            // figure out message size including header
            msgSize = Message.getMsgSize(m_recvBuf, bigEndian ) + msgHdrSize;

            __SECURITY_MD__(
            __SECURITY_MD_NOT_HMAC__(    // do not add MD len if the MD is for
            mdOffset = msgSize;
            msgSize += ORB.msgDigestLen;  // an HMAC (ie, no double counting)
            ))
                              
            __SECURITY_MAC__(
            macOffset = msgSize;
            msgSize += ORB.msgMacLen;
            )
                  
            __SECURITY__(
            //Where the padding starts...
            padOffset = msgSize;
            //msgSize should after this stmt have the larget msgSize
            msgSize = orb.cipherDecrypt.getOutputSize(msgSize-msgHdrSize) + msgHdrSize;
            )
            if (m_recvBuf.length < msgSize)
            {
               byte [] btmp = new byte [msgSize];
               System.arraycopy(m_recvBuf, 0, btmp, 0, msgHdrSize);
               m_recvBuf = btmp;
            }

            // read message after header
            in.readFully(m_recvBuf, msgHdrSize, msgSize-msgHdrSize);

            __DEBUG5__(//mqc.util.msgPrinter.printHex(m_recvBuf,0,msgSize);)
            __SECURITY__(
            //Reuse buffer m_recvBuf - works fine as long as it can hold the total size of the message!
            ORB.cipherDecrypt.doFinal(m_recvBuf,  msgHdrSize, msgSize-msgHdrSize, m_recvBuf, msgHdrSize);
            )
         } 
         __SECURITY__(
         catch (GeneralSecurityException gse) 
         {
            gse.printStackTrace();
         }
         )
         catch (Exception e) 
         {
            __DEBUG1__(
            e.printStackTrace();
            for (int i = 1;  i < nT;  i++)
            {
               System.err.println("sTimes: " + sTimes[i] + "  " + (sTimes[i] - sTimes[i-1]) + "  " + reqIDs[i]);
            }
            ) 
            break;
         }
         )
         __FIX_SIZE__(
         /* FIX_SIZE BEGIN */
         try
         {
            // Read the whole message
            // We trust that the buffer is large enough
            in.readFully(m_recvBuf, 0, msgSize);

            __DEBUG5__(//mqc.util.msgPrinter.printHex(m_recvBuf, 0, msgSize);)
            __SECURITY__(
            ORB.cipherDecrypt.doFinal(m_recvBuf, 0, msgSize, m_recvBuf, 0);     
            )
         }
         __SECURITY__(
         catch (GeneralSecurityException gse)
         {
            gse.printStackTrace();
         } 
         )
         catch (Exception e)
         {
            __DEBUG__(e.printStackTrace();)
            break;
         }
         /* FIX_SIZE END */
         )
               
         __SECURITY_MAC__(
         orb.msgMac.reset();
         orb.msgMac.update(m_recvBuf, 0, macOffset);
         byte [] compMAC = orb.msgMac.doFinal();
         byte [] recvMAC = new byte[orb.msgMacLen];
         System.arraycopy(m_recvBuf, macOffset, recvMAC, 0, orb.msgMacLen);
         if (!orb.msgMac.isEqual( compMAC, recvMAC))
         {
            System.err.println("MAC was NOT okay");
            continue;
         }
         )
         __SECURITY_MD__(
         __SECURITY_MD_NOT_HMAC__(
         orb.msgDigest.reset();
         orb.msgDigest.update(m_recvBuf, 0, mdOffset);
         byte [] compMD = orb.msgDigest.digest();
         byte [] recvMD = new byte[orb.msgDigestLen];
         System.arraycopy(m_recvBuf, mdOffset, recvMD, 0, orb.msgDigestLen);
         if (!orb.msgDigest.isEqual( compMD, recvMD) )
         {
            System.err.println("Message Digest was NOT okay");
            continue;
         }
         ))

         Request message = orb.findProtocolHandler(this, m_recvBuf );

         __DEBUG4__(
         //mqc.util.msgPrinter.printMessage(m_recvBuf, msgSize);
         )

         m_in.setBuffer( m_recvBuf );
         message.deMarshal( m_in );

         __GROUPCOM__(
         // GROUPCOM IS DEAD FOR NOW
         // make a copy of the data in the byte array
         byte[] actBuffer = new byte[msgSize];
         System.arraycopy(buffer, 0, actBuffer, 0, msgSize);
         try
         {
            String strBuffer = new String(actBuffer);
            __DEBUG1__( System.err.println("Buffer2String is: " + strBuffer);)
            // send message using groupcom to replicants
            orb.group.sendMessage(strBuffer,"message","","");
         }
         catch (Exception e)
         {
            // e.printStackTrace();
         }
         )
         __TEMPSPACE__(
         // next check the message request id
         int intReqID = message.getRequestID();
         // scan the buffer to see if this message has already been received
         if(intReqID > maxReqID)
         {
            // this is a non-duplicate message--process it
            __DEBUG1__( System.err.println("New ReqID: " + intReqID);)
            maxReqID = intReqID;

            // Let the server ORB serve request message
            msg = orb.serveRequest( message );

            try
            {
               // What is this ??
               Thread.sleep(0);
            }
            catch (Exception e)
            {
            }
            __DEBUG1__(System.err.println("Sr:" + (System.currentTimeMillis() % 1000));)
         }
         else
         {
            __DEBUG1__(System.err.println("Dup.ReqID: " + intReqID);)
         })
         __NOTEMPSPACE__(
         // Let the server ORB serve request message
         msg = orb.serveRequest( message );
         )

         if ( msg != null )
         {
            // Marshal Reply message on stream
            msg.marshal( m_out );

            // Send Reply message
            __VAR_SIZE__(boolean success = send( m_out.getBuffer(), m_out.bytesStreamed() );)
            __FIX_SIZE__(boolean success = send( m_out.getBuffer(), msgSize );)

            __DEBUG__(
            if ( ! success ) {
               System.err.println( "S_TCPHandler.run(): send was NOT successful." );
            }
            )
         }
      }

      orb.removeProtocolHandler(this);

      // hack to enforce the shutting down of the S_ORB
      // DO NOT COMMENT THESE LINES OUT---IT IS NEEDED ON THE SaJe PLATFORM!! -DM 
      if (!S_ORB.keepRunning)
      {
         S_ORB.shutDown(-1);
      }
   }

   /**
    * Sends a message. If you enabled fixed size messages; all messages will be transmitted by
    * that length over the network. The other option is to enable variable length packets. As
    * indicated by the name the transport layer sends the actual packet length over the network.
    * The only difference is that under variable length packets the GIOPHeader goes unencrypted
    * along the network wires. 
    *
    * @param byte[] The buffer to send
    * @param int The number of bytes to transmit
    * @return boolean true if data is sent successfully, false otherwise
    */
   public synchronized boolean send(byte[] buffer, int msgSize)
   {
      __DEBUG4__( System.err.println("S_TCPHandler::send -- buffer length: " + msgSize + " (" + Config.intPacketSize + ")");)
                
      __VAR_SIZE__(
      /* VAR_SIZE BEGIN */

      __SECURITY_MD__(
      __SECURITY_MD_NOT_HMAC__(    // do not add MD len if the MD is for
      mdOffset = msgSize;
      msgSize += ORB.msgDigestLen;  // an HMAC (ie, no double counting)
      ))

      __SECURITY_MAC__(
      macOffset = msgSize;
      msgSize += ORB.msgMacLen;
      )

      __SECURITY__(
      padOffset = msgSize;
      msgSize = orb.cipherDecrypt.getOutputSize(msgSize-msgHdrSize) + msgHdrSize;

      if (m_sendBuf == null || m_sendBuf.length < msgSize)
      {
         m_sendBuf = new byte [msgSize];
      }
      )
      /* VAR_SIZE END */
      )

      __DEBUG4__(
      //mqc.util.msgPrinter.printMessage(buffer, msgSize);
      )

      try
      {
         __SECURITY_MD__(
         __SECURITY_MD_NOT_HMAC__(
         try
         {
            orb.msgDigest.reset();
            orb.msgDigest.update(buffer, 0, mdOffset);
            orb.msgDigest.digest(buffer, mdOffset, orb.msgDigestLen);
         }
         catch (DigestException de)
         {
            de.printStackTrace();
         }
         ))

         __SECURITY_MAC__(
         try
         {
            orb.msgMac.reset();
                                orb.msgMac.update(buffer, 0, macOffset);
            orb.msgMac.doFinal(buffer, macOffset);
         }
         catch (ShortBufferException sbe)
         {
            sbe.printStackTrace();
         }
         catch (IllegalStateException ise)
         {
            ise.printStackTrace();
         }
         )
                        
         __SECURITY__(
         try
         {
            // Encrypt the buffer before sending it
            __FIX_SIZE__(
            ORB.cipherEncrypt.doFinal(buffer, 0, padOffset, m_sendBuf, 0);
            ) 
            __VAR_SIZE__(
            //Encrypt the message body and leave the header intact
            System.arraycopy(buffer, 0, m_sendBuf, 0, msgHdrSize);
            ORB.cipherEncrypt.doFinal(buffer, msgHdrSize, padOffset-msgHdrSize, m_sendBuf, msgHdrSize);
            )
         }
         catch (GeneralSecurityException gse)
         {
            gse.printStackTrace();
         }
         buffer = m_sendBuf;
         )
      } 
      catch (Exception e)
      {
         e.printStackTrace();
         return false;
      }
      try 
      {
         // now send the fixed length packet
         __TEMPORAL__(
         // use the temporal variable in the Config file to determine what the
         // level of temporal redundancy is
         for(int i=0;i<Config.temporal;i++)
         {
         )
            __DEBUG5__(//mqc.util.msgPrinter.printHex(buffer, 0, msgSize);)       
            out.write(buffer, 0, msgSize);
         __TEMPORAL__(
         }
         )
      }
      catch(IOException e)
      {
         e.printStackTrace();
         return false;
      }
      return true;
   }
}
