/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

include(`Config_macros.m4')

package mqc.transports;

import mqc.*;
import mqc.holders.*;
import mqc.ior.*;

import java.io.*;
import java.util.*;

__J2SE__(import java.net.*;)
__CLDC__(import javax.microedition.io.*;)

/**
 * UDP transport class
 *
 * @author Thor Egil Skaug    optimization
 * @author Eivind Naess       cldc support
 * @author Olav Haugan        initial version
 * @version 1.00
 */

public class S_UDPTransport extends S_Transport
{
   private TaggedProfile m_profiles[];                                     //supported profiles
   private String m_strLocalHost;
   private int m_nLocalPort;
   
   __J2SE__(
   private DatagramSocket m_socket;
   )
   __CLDC__(
   private DatagramConnection m_socket;
   )
   
   public S_UDPTransport(S_ORB orb)
   {
      super(orb);
      init();
   }
   
   public boolean init()
   {
      __J2SE__(
      try
      {
         m_socket = new DatagramSocket();                                  //this finds a new free port
         m_strLocalHost = InetAddress.getLocalHost().getHostAddress();
         m_nLocalPort = m_socket.getLocalPort();
      }
      catch(IOException ioe)
      {
         ioe.printStackTrace();
         return false;
      }
      )
      
      __CLDC__(
      m_strLocalHost = System.getProperty("localhost");                    //localport and localhost
      m_nLocalPort = Integer.parseInt(System.getProperty("localport"));    //must be set in environment
      String strConnURL = "datagram://" + ":" + m_nLocalPort;              //according to cldc documentation
      
      try
      {
         m_socket = (DatagramConnection)Connector.open(strConnURL, Connector.READ_WRITE, false);
      }
      catch(IOException ioe)
      {
         ioe.printStackTrace();
         return false;
      }
      )
      
      m_profiles = new TaggedProfile[Config.protocols.length];

      for (int i=0; i < m_profiles.length; i++)
      {
         switch (Config.protocols[i])
         {
         __GIOP_PROTOCOL__(
         case Config.GIOP:
            m_profiles[i] = new UIOPProfile();
            break;
         )
         __GIOPLITE_PROTOCOL__(
         case Config.GIOPLite:
            m_profiles[i] = new UIOPLiteProfile();
            break;
         )
         __MQCIOP_PROTOCOL__(
         case Config.MQCIOP:
            m_profiles[i] = new UMQCIOPProfile();
            break;
         )
         }
         m_profiles[i].major = (byte) 1;
         m_profiles[i].minor = (byte) 1;
         m_profiles[i].host = m_strLocalHost;
         m_profiles[i].port = m_nLocalPort;
      }         
      return true;
   }
   
   /**
    * This thread will listen and accept incoming datagram packets on a port, then
    * open a new port to handle client connections and use this to send
    * an ACK packet back to the client. The client will then use the new
    * port for further communications. One of the reasons for doing it this
    * way is that in CLDC we need to specify the listening port as an environment
    * variable, and we can therefore not open free ports as in TCP
    */   
   public void run()
   {
      byte [] data = new byte[__MAX_UDP_DATAGRAM_SIZE__];                  //tes: this should actually be changed to something rather short
      
      while(S_ORB.keepRunning)
      {
         __J2SE__(
         String strClientName = "";
         int nClientPort = 0;
         DatagramPacket packet = new DatagramPacket(data, data.length);
         DatagramSocket clientSocket;
         
         try
         {
            //receive connect packet from the client
            m_socket.receive(packet);
            
            //TBI if we wanted to be more complete we should send an ACK or NACK
            //back depending on what kind of UDP types we are offering
                  
            //retrive client's information from the packet
            strClientName  = packet.getAddress().getHostName();
            nClientPort = packet.getPort();
   
            //create a datagram socket for this client
            clientSocket = new DatagramSocket();
   
            //send a packet to the client using this datagram socket
            clientSocket.send(new DatagramPacket(data, data.length, InetAddress.getByName(strClientName), nClientPort));
         }
         catch(IOException ioe)
         {
            ioe.printStackTrace();
            return;
         }
         )
         __CLDC__(
         String strClientURL = "";
         Datagram dataGram = null;
         DatagramConnection clientSocket = null;
         try
         {
            dataGram = m_socket.newDatagram(data, data.length);               //tes: this should actually be changed to something rather short
            m_socket.receive(dataGram);                                       //receive a connect packet from client

            //TBI if we wanted to be more complete we should send an ACK or NACK
            //back depending on what kind of UDP types we are offering
         
            //retrieve client info
            strClientURL = dataGram.getAddress();
            clientSocket = (DatagramConnection)Connector.open(strClientURL, Connector.READ_WRITE, false);
            
            //send a packet to the client using this datagram socket
            clientSocket.send(dataGram);
         }
         catch(IOException ioe)
         {
            ioe.printStackTrace();   
         }
         )
         //start the correct handler
         switch((int)(data[0]))
         {
            __UNRELIABLE_UDP__(
            case Config.UDP:
               __J2SE__(
               new S_UDPHandler(clientSocket, strClientName, nClientPort, orb);
               )
               __CLDC__(
               new S_UDPHandler(clientSocket, strClientURL, orb);
               )
               break;
            )
            __GO_BACK_N_UDP__(
            case Config.Go_Back_N_UDP:
               __J2SE__(
               new S_UDPGoBackNHandler(clientSocket, strClientName, nClientPort, orb);
               )
               __CLDC__(
               new S_UDPGoBackNHandler(clientSocket, strClientURL, orb);
               )
               break;
            )
            __STOP_N_WAIT_UDP__(
            case Config.Stop_N_Wait_UDP:
               __J2SE__(
               new S_UDPStopNWaitHandler(clientSocket, strClientName, nClientPort, orb);
               )
               __CLDC__(
               new S_UDPStopNWaitHandler(clientSocket, strClientURL, orb);
               )
               break;
            )
         }
      }
      if(!S_ORB.keepRunning)
      {
         S_ORB.shutDown(-1);
      }
   }
   
   public TaggedProfile[] getTaggedProfile()
   {
      return m_profiles;   
   }      
}
