/*
 * Copyright (c) 2003 David E. Bakken, his research students, and Washington State University.
 * Please see the file LICENSE.pdf for more details on terms of use.
 */

include(`Config_macros.m4')

package mqc.transports;

import mqc.*;
import mqc.holders.*;
import mqc.protocols.*;
import java.io.*;
import java.util.*;

/**
* Base class for transport layers
* 
* @author Olav Haugan
* @author David McKinnon
*/
public abstract class Transport
{
   private static int reqID;

   protected CDRInputStream m_in;
   protected CDROutputStream m_out;

  /**
   * Constructor. Initializes variables.
   */
   public Transport()
   {
      reqID = 0;
   }

  /**
   * Constructor. Initializes variables.
   */
   public Transport(Delegate delegate)
   {
      reqID = 0;
   }

  /**
   * Initializes a connection
   */
   public abstract boolean init();

  /**
   * Sends a message
   *
   * @param byte[] The buffer to send
   * @param int The number of bytes to transmit
   */
   public abstract boolean send(byte[] buffer, int length);

   public void send( mqc.protocols.Request req )
   {
      req.marshal( m_out );
      send( m_out.getBuffer(), m_out.bytesStreamed() );
   }

  /**
   * Waits for a message and receives it. This
   * function alters the msg object! Use 
   * msg.getBuffer(), msg.getEndianess()!
   *
   * @param rep  The reply message to deMarshal into
   * @return  whether trasport successfully received and deMarshaled
   *          a Reply message.
   */
   public abstract boolean waitForReply( mqc.protocols.Reply rep );

   __TEMPSPACE__(
   /**
    * Skips all buffered reply messages.
    *
    * @param rep  the Reply message which this method can use to demarhal a temporary 
    *             message into.
    * @return  0 if no reply skipped, 
    *          otherwise the number of bytes skipped
    */
   public abstract int skipReply( Reply rep );
   )

  /**
   * Get next request ID
   *
   * @return int The next ID
   */
   public int getNextRequestID()
   {
      return reqID++;
   }

}
